# coding=utf-8

import time
import socket
import threading

from pymycobot.mercury_api import MercuryCommandGenerator
from pymycobot.error import calibration_parameters


class MercurySocket(MercuryCommandGenerator):
    def __init__(self, ip, netport=9000, debug=False):
        """
        Args:
            ip: Server ip
            netport: Server port(default 9000)
        """
        super(MercurySocket, self).__init__(debug)
        self.calibration_parameters = calibration_parameters
        self.SERVER_IP = ip
        self.SERVER_PORT = netport
        self.sock = self.connect_socket()
        self.lock = threading.Lock()
        self.read_threading = threading.Thread(target=self.read_thread, args=("socket", ))
        self.read_threading.daemon = True
        self.read_threading.start()

    def connect_socket(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((self.SERVER_IP, self.SERVER_PORT))
        return sock
        
    def open(self):
        self.sock = self.connect_socket()
        
    def close(self):
        self.sock.close()



# coding: utf-8

import threading

from pymycobot.Interface import MyBuddyCommandGenerator
from pymycobot.common import ProtocolCode, write, read
from pymycobot.error import calibration_parameters
from pymycobot.bluet import BluetoothConnection


class MyBuddyBlueTooth(MyBuddyCommandGenerator):
    """MyBuddy bluetooth API"""
    _write = write
    def __init__(self, bt_address=None, port = 10):
        """When bt_address is the default value of None, enter the Bluetooth search mode. There is a default Bluetooth search time of 5 seconds"""
        super(MyBuddyBlueTooth).__init__()
        self.ble = BluetoothConnection(bt_address, port)
        self.sock = self.ble.connect_target_device()
        self.lock = threading.Lock()
        
    def connect(self, serialport="/dev/ttyAMA0", baudrate="1000000", timeout='0.2'):
        """Connect the robot arm through the serial port and baud rate
        
        Args:
            serialport: (str) default /dev/ttyAMA0
            baudrate: default 1000000
            timeout: default 0.1
        
        """
        self.rasp = True
        self._write(serialport, "socket")
        self._write(baudrate, "socket")
        self._write(timeout, "socket")
        # self._write([254, 254, 1, 2, 32, 32], "socket")
        # self._write([254, 254, 1, 2, 32, 32], "socket")
        # self._write([254, 254, 1, 2, 32, 32], "socket")
        
        # self._write(timeout, "socket")
        # self._write(timeout, "socket")
        
        
    def _mesg(self, genre, *args, **kwargs):
        """

        Args:
            genre: command type (Command)
            *args: other data.
                   It is converted to octal by default.
                   If the data needs to be encapsulated into hexadecimal,
                   the array is used to include them. (Data cannot be nested)
            **kwargs: support `has_reply`
                has_reply: Whether there is a return value to accept.
        """
        real_command, has_reply = super(
            MyBuddyBlueTooth, self)._mesg(genre, *args, **kwargs)
        with self.lock:
            data = self._write(self._flatten(real_command), "socket")

            if data:
                res = self._process_received(data, genre, arm=12)
                if res == []:
                    return None
                if genre in [
                    ProtocolCode.ROBOT_VERSION,
                    ProtocolCode.SOFTWARE_VERSION,
                    ProtocolCode.GET_ROBOT_ID,
                    ProtocolCode.IS_POWER_ON,
                    ProtocolCode.IS_CONTROLLER_CONNECTED,
                    ProtocolCode.IS_PAUSED,  # TODO have bug: return b''
                    ProtocolCode.IS_IN_POSITION,
                    ProtocolCode.IS_MOVING,
                    ProtocolCode.IS_SERVO_ENABLE,
                    ProtocolCode.IS_ALL_SERVO_ENABLE,
                    ProtocolCode.GET_SERVO_DATA,
                    ProtocolCode.GET_DIGITAL_INPUT,
                    ProtocolCode.GET_GRIPPER_VALUE,
                    ProtocolCode.IS_GRIPPER_MOVING,
                    ProtocolCode.GET_SPEED,
                    ProtocolCode.GET_ENCODER,
                    ProtocolCode.GET_BASIC_INPUT,
                    ProtocolCode.GET_TOF_DISTANCE,
                    ProtocolCode.GET_END_TYPE,
                    ProtocolCode.GET_MOVEMENT_TYPE,
                    ProtocolCode.GET_REFERENCE_FRAME,
                    ProtocolCode.GET_FRESH_MODE,
                    ProtocolCode.SetHTSGripperTorque,
                    ProtocolCode.GetHTSGripperTorque,
                    ProtocolCode.GetGripperProtectCurrent,
                    ProtocolCode.InitGripper,
                    ProtocolCode.SET_FOUR_PIECES_ZERO
                    # ProtocolCode.GET_SERVO_CURRENTS
                ]:
                    return self._process_single(res)
                elif genre in [ProtocolCode.GET_ANGLES]:
                    return [self._int2angle(angle) for angle in res]
                elif genre in [ProtocolCode.GET_ANGLE]:
                    return self._int2angle(res[0]) if res else None
                elif genre in [ProtocolCode.GET_COORD]:
                    if real_command[5] < 4:
                        if real_command[2] == 3:
                            return self._int2angle(res[0]) if res else None
                        return self._int2coord(res[0]) if res else None
                    else:
                        return self._int2angle(res[0]) if res else None
                elif genre in [ProtocolCode.GET_ALL_BASE_COORDS, ProtocolCode.GET_COORDS, ProtocolCode.GET_TOOL_REFERENCE, ProtocolCode.GET_WORLD_REFERENCE, ProtocolCode.GET_BASE_COORDS, ProtocolCode.GET_BASE_COORD, ProtocolCode.BASE_TO_SINGLE_COORDS]:
                    if res:
                        r = [] 
                        for idx in range(3):
                            r.append(self._int2coord(res[idx]))
                        for idx in range(3, 6):
                            r.append(self._int2angle(res[idx]))
                        if len(res) == 12:
                            r1 = []
                            for idx in range(6, 9):
                                r1.append(self._int2coord(res[idx]))
                            for idx in range(9, 12):
                                r1.append(self._int2angle(res[idx]))
                            return [r, r1]
                        return r
                    else:
                        return res
                elif genre in [ProtocolCode.GET_JOINT_MAX_ANGLE, ProtocolCode.GET_JOINT_MIN_ANGLE]:
                    return self._int2coord(res[0])
                elif genre in [ProtocolCode.GET_SERVO_VOLTAGES, ProtocolCode.COLLISION]:
                    return [self._int2coord(angle) for angle in res]
                else:
                    return res
            return None
        
    def close(self):
        self._write("close","socket")
        self.sock.close()
        



import socket
import json
import threading
import struct

class MercuryChassisError(Exception):
    pass

class MercuryChassis:
    """
    Mercury X1 mobile chassis car socket class
    """
    def __init__(self, ip=None):
        self.ifname = b"wlan0"
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if ip is not None:
            self.host = ip
        else:
            import fcntl
            try:
                self.host = socket.inet_ntoa(fcntl.ioctl(self.server_socket.fileno(), 0x8915, struct.pack('256s', self.ifname[:15]))[20:24])  #IP
            except:
                self.host = "127.0.0.1"
        self._sock.connect((self.host, 9999))
        self.recv = threading.Thread(target=self.check_move_end, daemon=True)
        self.recv.start()
        self.move_end = False
        
    def close(self):
        self._sock.close()
    
    def open(self):
        self._sock.connect((self.host, 9999))
        
    def check_move_end(self):
        while True:
            try:
                data = self._sock.recv(1024)
                data = json.loads(data)
                self.move_end = data
            except:
                pass
    
    # @property
    def is_move_end(self):
        """Is the movement over

        Returns:
            _type_: _description_
        """
        return self.move_end
        
    def go_straight(self, speed=0.25, exercise_duration=5):
        """Forward control

        Args:
            speed (float, optional): Movement speed. Defaults to 0.25. range 0 ~ 1
            exercise_duration (int, optional): Exercise duration. Defaults to 5s.
        """
        if speed < 0 or speed > 1:
            raise MercuryChassisError("The movement speed range is 0~1, but the received value is {}".format(speed))
        command = {"goStraight": {"time": exercise_duration, "speed": speed}}
        self._sock.sendall(json.dumps(command).encode())
        
    def go_back(self, speed=0.25, exercise_duration=5):
        """Back control

        Args:
            speed (float, optional): Movement speed. Defaults to 0.25. range 0 ~ 1
            exercise_duration (int, optional): Exercise duration. Defaults to 5s.
        """
        if speed < 0 or speed > 1:
            raise MercuryChassisError("The movement speed range is 0~1, but the received value is {}".format(speed))
        command = {"goBack": {"time": exercise_duration, "speed": speed}}
        self._sock.sendall(json.dumps(command).encode())
        
    def turn_left(self, speed=0.5, exercise_duration=5):
        """left turn control

        Args:
            speed (float, optional): Movement speed. Defaults to 0.25. range 0 ~ 1
            exercise_duration (int, optional): Exercise duration. Defaults to 5s.
        """
        if speed < 0 or speed > 1:
            raise MercuryChassisError("The movement speed range is 0~1, but the received value is {}".format(speed))
        command = {"turnLeft": {"time": exercise_duration, "speed": speed}}
        self._sock.sendall(json.dumps(command).encode())
        
    def turn_right(self, speed=0.5, exercise_duration=5):
        """_summary_

        Args:
            speed (float, optional): Movement speed. Defaults to 0.25. range 0 ~ 1
            exercise_duration (int, optional): Exercise duration. Defaults to 5s.
        """
        if speed < 0 or speed > 1:
            raise MercuryChassisError("The movement speed range is 0~1, but the received value is {}".format(speed))
        command = {"turnRight": {"time": exercise_duration, "speed": speed}}
        self._sock.sendall(json.dumps(command).encode())
        
    def stop(self):
        """stop motion"""
        command = {"stop": True}
        self._sock.sendall(json.dumps(command).encode())

    def init_position(self,position_x,position_y,orientation_z,orientation_w,covariance):
        """Set navigation starting position

        Args:
            position_x (_type_): _description_
            position_y (_type_): _description_
            orientation_z (_type_): _description_
            orientation_w (_type_): _description_
            covariance (_type_): _description_
        """
        command = {"initPosition": {"x": position_x, "y": position_y, "o_z": orientation_z, "o_w": orientation_w, "cov": covariance}}
        self._sock.sendall(json.dumps(command).encode())

    def goto_position(self,position_x,position_y,orientation_z,orientation_w):
        """Set navigation target location

        Args:
            position_x (_type_): _description_
            position_y (_type_): _description_
            orientation_z (_type_): _description_
            orientation_w (_type_): _description_
        """
        command = {"goToPosition": {"x": position_x, "y": position_y, "o_z": orientation_z, "o_w": orientation_w}}
        self._sock.sendall(json.dumps(command).encode())

    def cancel_navigation(self):
        """Cancel navigation
        """
        command = {"movebaseCancel": True}
        self._sock.sendall(json.dumps(command).encode())        

    def get_software_version(self):
        """Get the base server version number"""
        command = {"getSoftWareVersion": True}
        self._sock.sendall(json.dumps(command).encode()) 
        while True:
            if self.move_end:
                data = self.move_end.get("getSoftWareVersion", None)
                if data:
                    return self.move_end["getSoftWareVersion"]["return"]

    def get_base_ros_version(self):
        """Get the base ROS project version number"""
        command = {"getBaseROSVersion": True}
        self._sock.sendall(json.dumps(command).encode()) 
        while True:
            if self.move_end:
                data = self.move_end.get("getBaseROSVersion", None)
                if data:
                    return self.move_end["getBaseROSVersion"]["return"]

    def get_battery_state(self):
        """Get battery level
        """
        command = {"batteryState": True}
        self._sock.sendall(json.dumps(command).encode())
        while True:
            if self.move_end:
                data = self.move_end.get("batteryState", None)
                if data:
                    return self.move_end["batteryState"]["return"]
        

        


# coding=utf-8
from __future__ import division

import threading


from pymycobot.common import ProtocolCode
from pymycobot.myarm_api import MyArmAPI


class MyArmM(MyArmAPI):

    def __init__(self, port, baudrate="115200", timeout=0.1, debug=False):
        super(MyArmM, self).__init__(port, baudrate, timeout,debug)

    def set_joint_angle(self, joint_id, angle, speed):
        """Sets the individual joints to move to the target angle

        Args:
            joint_id (int) : 0 - 254
            angle (int) : 0 - 254
            speed (int) : 1 - 100
        """
        self._mesg(ProtocolCode.SEND_ANGLE, joint_id, [self._angle2int(angle)], speed)

    def set_joints_angle(self, angles, speed):
        """Sets all joints to move to the target angle

        Args:
            angles (list[int]):  0 - 254
            speed (int): 0 - 100
        """
        angles = list(map(self._angle2int, angles))
        return self._mesg(ProtocolCode.SEND_ANGLES, angles, speed)

    def is_robot_moving(self):
        """See if the robot is moving

        Returns:
            1: moving
            0: not moving
        """
        return self._mesg(ProtocolCode.IS_MOVING, has_reply=True)

    def stop_robot(self):
        """The robot stops moving"""
        self._mesg(ProtocolCode.STOP)

    # def is_in_position(self):
    #     """Whether the robot has reached the specified point
    #     Returns:
    #
    #     """
    #     return self._mesg(ProtocolCode.IS_IN_POSITION, reply=True)

    def set_servo_encoder(self, servo_id, encoder, speed):
        """Sets the individual motor motion to the target encoder potential value

        Args:
            servo_id: (int) 0 - 254
            encoder: (int) 0 - 4095
            speed: (int) 1 - 100

        """
        self._mesg(ProtocolCode.SET_ENCODER, servo_id, [encoder], speed)

    def set_servos_encoder(self, positions, speed):
        """Set the encoder potential value for multiple motors moving to the target

        Args:
            positions (list[int * 8]): 0 - 4095:
            speed (int): 1 - 100:
        """
        self._mesg(ProtocolCode.SET_ENCODERS, positions, speed)

    def set_servos_encoder_drag(self, encoders, speeds):
        """Set multiple servo motors with a specified speed to the target encoder potential value"""
        self.calibration_parameters(class_name=self.__class__.__name__, encoders=encoders, speeds=speeds)
        self._mesg(ProtocolCode.SET_ENCODERS_DRAG, encoders, speeds)

    def set_assist_out_io_state(self, io_number, status=1):
        """Set the auxiliary pin status

        Args:
            io_number: 1 - 6
            status: 0/1; 0: low; 1: high. default: 1

        """
        self._mesg(ProtocolCode.SET_AUXILIARY_PIN_STATUS, io_number, status)

    def get_assist_in_io_state(self, io_number):
        """Get the auxiliary pin status

        Args:
            io_number (int): 1 - 6

        Returns:
            0 or 1. 1: high 0: low

        """
        return self._mesg(ProtocolCode.GET_AUXILIARY_PIN_STATUS, io_number, has_reply=True)

